import contextlib
import dataclasses
import inspect
import typing
from typing import Dict, Iterable, Union

from django.conf import settings as django_settings
from django.contrib.auth.models import User
from django.core import signing
from strawberry import auto
from strawberry.annotation import StrawberryAnnotation
from strawberry.arguments import StrawberryArgument
from strawberry.types import Info
from strawberry.unset import UNSET
from strawberry.utils.str_converters import to_camel_case
from strawberry_django_jwt.exceptions import JSONWebTokenError

from .exceptions import TokenScopeError, WrongUsage


def hide_args_kwargs(field):
    sig = inspect.signature(field)
    cleared = tuple(p for p in sig.parameters.values() if p.name not in ("kwargs", "args"))
    field.__signature__ = inspect.signature(field).replace(parameters=(cleared))
    return field


def isiterable(value):
    try:
        iter(value)
    except TypeError:
        return False
    return True


def camelize(data):
    if isinstance(data, dict):
        return {to_camel_case(k): camelize(v) for k, v in data.items()}
    if isiterable(data) and not isinstance(data, str):
        return [camelize(d) for d in data]
    return data


def list_to_dict(lst: [str]):
    """takes list of string and creates a dict with str as their values"""
    new_dict = {}
    for item in lst:
        new_dict[item] = str
    return new_dict


def get_request(info: Info):
    if hasattr(info.context, "user"):
        return info.context
    return info.context.request


def g_user(info: Info) -> User:
    # returns a user from info obj
    user = getattr(info.context, "user", False)
    if user:
        return user
    return info.context.request.user


def get_token(user, action, **kwargs):
    username = user.get_username()
    if hasattr(username, "pk"):
        username = username.pk
    payload = {user.USERNAME_FIELD: username, "action": action}
    if kwargs:
        payload.update(**kwargs)
    token = signing.dumps(payload)
    return token


def get_payload_from_token(token, action, exp=None):
    payload = signing.loads(token, max_age=exp)
    _action = payload.pop("action")
    if _action != action:
        raise TokenScopeError
    return payload


def using_refresh_tokens():
    if (
        hasattr(django_settings, "GRAPHQL_JWT")
        and django_settings.GRAPHQL_JWT.get("JWT_LONG_RUNNING_REFRESH_TOKEN", False)
        and "strawberry_django_jwt.refresh_token" in django_settings.INSTALLED_APPS
    ):
        return True
    return False


def revoke_user_refresh_token(user):
    if using_refresh_tokens():
        refresh_tokens = user.refresh_tokens.all()
        for refresh_token in refresh_tokens:
            with contextlib.suppress(JSONWebTokenError):
                refresh_token.revoke()


def flat_dict(dict_or_list):
    """
    if is dict, return list of dict keys,
    if is list, return the list
    """
    return list(dict_or_list.keys()) if isinstance(dict_or_list, dict) else dict_or_list


def normalize_fields(dict_or_list, extra_list_or_dict):
    """
    helper merge settings defined filed
    with default str type
    """
    if not isinstance(extra_list_or_dict, dict):
        extra_list_or_dict = list_to_dict(extra_list_or_dict)
    if not isinstance(dict_or_list, dict):
        dict_or_list = list_to_dict(dict_or_list)
    dict_or_list.update(extra_list_or_dict)

    return dict_or_list


def create_strawberry_argument(python_name: str, graphql_name: str, type_, default=None):
    return StrawberryArgument(
        python_name=python_name,
        graphql_name=graphql_name,
        type_annotation=StrawberryAnnotation(type_),
        default=default or UNSET,
    )


def inject_fields(fields: Union[Dict[str, type], Iterable[str]]):
    """
    Injects the supplied fields to the decorated class.
    If the given fields are list they would be annotated with `auto`
    """

    def wrapped(cls):
        annotations = list(cls.__annotations__.items())
        res = fields
        if isinstance(fields, Iterable):
            # Checking that the field is not a "" redundant string
            # This is quite common behavior with django's user model that a user override
            res = {field: auto for field in fields if field}

        elif not isinstance(fields, dict):
            raise WrongUsage(
                "Can handle only list of strings or dict of name and types."
                f"You provided {type(fields)}"
            )
        # this solves non default fields after default fields
        annotations.extend(list(res.items()))
        annotations.reverse()
        annotations = {name: annotation for name, annotation in annotations}
        cls.__annotations__ = annotations
        return cls

    return wrapped


def inject_many(fields: Iterable[Union[Dict[str, type], Iterable[str]]]):
    """
    Injects the supplied iterables to the decorated class.
    """

    def wrapped(cls):
        for node in fields:
            inject_fields(node)(cls)
        return cls

    return wrapped


def make_dataclass_helper(required: Union[dict, list], non_required: Union[dict, list]):
    res_req = []
    res_non_req = []

    if isinstance(required, dict):
        for key in required:
            res_req.append((to_camel_case(key), required[key]))

    elif isinstance(required, list):
        for key in required:
            res_req.append((to_camel_case(key), str))

    if isinstance(non_required, dict):
        for key in non_required:
            res_non_req.append(
                (
                    to_camel_case(key),
                    typing.Optional[non_required[key]],
                    dataclasses.field(default=None),
                )
            )

    elif isinstance(non_required, list):
        for key in non_required:
            res_non_req.append(
                (
                    to_camel_case(key),
                    typing.Optional[str],
                    dataclasses.field(default=None),
                )
            )

    return res_req + res_non_req


def is_optional(field):
    """
    whether strawberry field is optional or not
    """
    return typing.get_origin(field) is Union and type(None) in typing.get_args(field)



from dataclasses import dataclass, field
from datetime import timedelta
from random import SystemRandom
from typing import Any, Callable, NewType, Union

from django.conf import settings as django_settings
import strawberry


def default_text_factory():
    return "".join(
        [
            str(node)
            for node in [
                SystemRandom().randint(0, 10) for _ in range(5, SystemRandom().randint(10, 20))
            ]
        ]
    )


_USER_NODE_FILTER_FIELDS = {
    "email": ["exact"],
    "username": ["exact", "icontains", "istartswith"],
    "is_active": ["exact"],
    "status__archived": ["exact"],
    "status__verified": ["exact"],
    "status__secondary_email": ["exact"],
}

DjangoSetting = NewType("DjangoSetting", Union[dict, list, str, Any])


@dataclass
class GqlAuthSettings:
    # if allow logging in without verification,
    # the register mutation will return a token
    ALLOW_LOGIN_NOT_VERIFIED: bool = False
    # mutation fields options
    LOGIN_OPTIONAL_FIELDS: list = field(default_factory=lambda: [])
    LOGIN_REQUIRE_CAPTCHA: bool = True
    LOGIN_REQUIRED_FIELDS: Union[dict, list] = field(
        default_factory=lambda: ["username", "password"]
    )
    """
    These fields would be used to authenticate with SD-jwt `authenticate` function.
    This function will call each of our `AUTHENTICATION_BACKENDS`,
    And will return the user from one of them unless `PermissionDenied` was raised.
    You can pass any fields that would be accepted by your backends.
    """

    REGISTER_MUTATION_FIELDS: Union[dict, list] = field(
        default_factory=lambda: ["email", "username"]
    )
    """
    required fields on register, plus password1 and password2,
    can be a dict like UPDATE_MUTATION_fieldS setting
    """
    REGISTER_MUTATION_FIELDS_OPTIONAL: list = field(default_factory=lambda: [])
    REGISTER_REQUIRE_CAPTCHA: bool = True
    # captcha stuff
    CAPTCHA_EXPIRATION_DELTA: timedelta = timedelta(seconds=120)
    CAPTCHA_MAX_RETRIES: int = 5
    CAPTCHA_TEXT_FACTORY: Callable = default_text_factory
    """
    A callable with no arguments that returns a string.
    This will be used to generate the captcha image.
    """
    CAPTCHA_TEXT_VALIDATOR: Callable[[str, str], bool] = (
        lambda original, received: original == received
    )
    """
    A callable that will receive the original string vs user input and returns a boolean.
    """
    FORCE_SHOW_CAPTCHA: bool = False
    """
    Whether to show the captcha image after it has been created for debugging purposes.
    """
    CAPTCHA_SAVE_IMAGE: bool = False
    """
    if True, an png representation of the captcha will be saved under
    MEDIA_ROOT/captcha/<datetime>/<uuid>.png
    """
    # optional fields on update account, can be list of fields
    UPDATE_MUTATION_FIELDS: Union[dict, list] = field(
        default_factory=lambda: {"first_name": str, "last_name": str}
    )

    # email tokens
    EXPIRATION_ACTIVATION_TOKEN: timedelta = timedelta(days=7)
    EXPIRATION_PASSWORD_RESET_TOKEN: timedelta = timedelta(hours=1)
    EXPIRATION_SECONDARY_EMAIL_ACTIVATION_TOKEN: timedelta = timedelta(hours=1)
    EXPIRATION_PASSWORD_SET_TOKEN: timedelta = timedelta(hours=1)
    # email stuff
    EMAIL_FROM: DjangoSetting = lambda: getattr(
        django_settings, "DEFAULT_FROM_EMAIL", "test@email.com"
    )
    SEND_ACTIVATION_EMAIL: bool = True
    # client: example.com/activate/token
    ACTIVATION_PATH_ON_EMAIL: str = "activate"
    ACTIVATION_SECONDARY_EMAIL_PATH_ON_EMAIL: str = "activate"
    # client: example.com/password-set/token
    PASSWORD_SET_PATH_ON_EMAIL: str = "password-set"
    # client: example.com/password-reset/token
    PASSWORD_RESET_PATH_ON_EMAIL: str = "password-reset"
    # email subjects templates
    EMAIL_SUBJECT_ACTIVATION: str = "email/activation_subject.txt"
    EMAIL_SUBJECT_ACTIVATION_RESEND: str = "email/activation_subject.txt"
    EMAIL_SUBJECT_SECONDARY_EMAIL_ACTIVATION: str = "email/activation_subject.txt"
    EMAIL_SUBJECT_PASSWORD_SET: str = "email/password_set_subject.txt"
    EMAIL_SUBJECT_PASSWORD_RESET: str = "email/password_reset_subject.txt"
    # email templates
    EMAIL_TEMPLATE_ACTIVATION: str = "email/activation_email.html"
    EMAIL_TEMPLATE_ACTIVATION_RESEND: str = "email/activation_email.html"
    EMAIL_TEMPLATE_SECONDARY_EMAIL_ACTIVATION: str = "email/activation_email.html"
    EMAIL_TEMPLATE_PASSWORD_SET: str = "email/password_set_email.html"
    EMAIL_TEMPLATE_PASSWORD_RESET: str = "email/password_reset_email.html"
    EMAIL_TEMPLATE_VARIABLES: dict = field(default_factory=lambda: {})
    # query stuff
    USER_NODE_EXCLUDE_FIELDS: Union[dict, list] = field(
        default_factory=lambda: ["password", "is_superuser"]
    )
    USER_NODE_FILTER_FIELDS: dict = field(default_factory=lambda: _USER_NODE_FILTER_FIELDS)
    # others
    # turn is_active to False instead
    ALLOW_DELETE_ACCOUNT: bool = False
    # mutation error type
    CUSTOM_ERROR_TYPE: strawberry.scalar = "gqlauth.bases.scalars.ExpectedErrorType"
    # registration with no password
    ALLOW_PASSWORDLESS_REGISTRATION: bool = False
    SEND_PASSWORD_SET_EMAIL: bool = False

    def __post_init__(self):
        # if there override the defaults
        if "email" not in self.REGISTER_MUTATION_FIELDS:
            self.SEND_ACTIVATION_EMAIL = False



from typing import Optional

from django.contrib.auth import get_user_model
import strawberry
from strawberry import auto
from strawberry.types import Info

from gqlauth import models
from gqlauth.settings import gqlauth_settings
from gqlauth.utils import inject_many

USER_MODEL = get_user_model()
user_pk_field = USER_MODEL._meta.pk.name

USER_FIELDS = [
    [
        user_pk_field,
        USER_MODEL.USERNAME_FIELD,
        USER_MODEL.EMAIL_FIELD,
    ],
    gqlauth_settings.UPDATE_MUTATION_FIELDS,
]


@strawberry.django.filters.filter(models.UserStatus)
class UserStatusFilter:
    verified: auto
    archived: auto
    secondary_email: auto


@strawberry.django.type(model=models.UserStatus, filters=UserStatusFilter)
class UserStatusType:
    verified: auto
    archived: auto
    secondary_email: auto


@strawberry.django.filters.filter(USER_MODEL)
@inject_many(USER_FIELDS)
class UserFilter:
    logentry_set: auto
    is_superuser: auto
    last_login: auto
    is_staff: auto
    is_active: auto
    date_joined: auto
    status: UserStatusFilter


@strawberry.django.type(model=USER_MODEL, filters=UserFilter)
@inject_many(USER_FIELDS)
class UserType:
    logentry_set: auto
    is_superuser: auto
    last_login: auto
    is_staff: auto
    is_active: auto
    date_joined: auto
    status: UserStatusType

    @strawberry.django.field
    def archived(self, info: Info) -> bool:
        return self.status.archived

    @strawberry.django.field
    def verified(self, info: Info) -> bool:
        return self.status.verified

    @strawberry.django.field
    def secondary_email(self, info: Info) -> Optional[str]:
        return self.status.secondary_email



from base64 import b64decode, b64encode
import typing

from PIL.Image import Image
import strawberry

image = strawberry.scalar(
    typing.NewType("image", Image),
    serialize=lambda v: b64encode(v).decode("ascii"),
    parse_value=lambda v: b64decode(v),
)



# Generated by Django 4.0.5 on 2022-06-19 05:03

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0002_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="captcha",
            name="id",
        ),
        migrations.AlterField(
            model_name="captcha",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, primary_key=True, serialize=False
            ),
        ),
    ]



# Generated by Django 4.0.6 on 2022-08-02 19:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0003_remove_captcha_id_alter_captcha_uuid"),
    ]

    operations = [
        migrations.AddField(
            model_name="captcha",
            name="url",
            field=models.ImageField(
                default=None,
                editable=False,
                help_text="url for the captcha image",
                upload_to="captcha/%Y/%m/%d/",
            ),
        ),
    ]



# Generated by Django 4.0.6 on 2022-08-02 19:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0004_captcha_url"),
    ]

    operations = [
        migrations.RenameField(
            model_name="captcha",
            old_name="url",
            new_name="image",
        ),
    ]



import dataclasses
import typing

from asgiref.sync import sync_to_async
from django.contrib.auth import get_user_model
import strawberry
from strawberry.types import Info

from gqlauth.bases.exceptions import WrongUsage
from gqlauth.bases.interfaces import OutputInterface
from gqlauth.bases.scalars import ExpectedErrorType
from gqlauth.utils import (
    create_strawberry_argument,
    hide_args_kwargs,
    is_optional,
    list_to_dict,
    make_dataclass_helper,
)

UserModel = get_user_model()


class DynamicInputMixin:
    """
    A class that knows how to initialize strawberry input

    as dict { arg name: arg type }
    as list [arg_name,] -> defaults to String

    ### usage:
        class SomeMutation(DynamicInputMixin, MutationMixin):
                class _meta:
                    _inputs = [some, non, required, fields]
                    _required_inputs = {some:str, required:int, fields:list}

                def resolve_mutation(self, input):
                    logic...

    """

    def __init_subclass__(cls, **kwargs):
        _inputs = getattr(cls._meta, "_inputs", [])
        _required_inputs = getattr(cls._meta, "_required_inputs", [])

        if _inputs or _required_inputs:
            if not isinstance(_inputs, (dict, list)) and _inputs:
                raise WrongUsage(f"dynamic inputs can be list or dict not{type(_inputs)}")
            if not isinstance(_required_inputs, (dict, list)) and _required_inputs:
                raise WrongUsage(
                    f"dynamic required inputs can be list or dict" f" not{type(_required_inputs)}"
                )

        parent_resolver_name = getattr(cls._meta, "_parent_resolver_name", None)
        if parent_resolver_name:
            _inputs = list_to_dict(_inputs)
            _required_inputs = list_to_dict(_required_inputs)

            parent_resolver = getattr(cls, parent_resolver_name, None)
            for name, ret_type in parent_resolver.base_resolver.annotations.items():
                if name != "return":
                    if is_optional(ret_type):
                        _inputs[name] = typing.get_args(ret_type)[0]
                    else:
                        _required_inputs[name] = ret_type

        if _inputs or _required_inputs:
            dc = dataclasses.make_dataclass(
                f"{cls.__name__}Input",
                [
                    *make_dataclass_helper(_required_inputs, _inputs),
                ],
            )

            inputs = strawberry.input(dc)
            cls._meta.inputs = inputs
        else:
            cls._meta.inputs = None
        super().__init_subclass__()

    def field(self):
        raise NotImplementedError()

    async def afield(self):
        raise NotImplementedError()


class DynamicArgsMixin:
    """
    A class that knows how to initialize strawberry arguments

    as dict { arg name: arg type }
    as list [arg_name,]  defaults to String

    ### usage:
        class SomeMutation(DynamicInputMixin, MutationMixin):
                class _meta:
                    _inputs = [some, non, required, fields]
                    _required_inputs = {some:str, required:int, fields:list}

                def resolve_mutation(self, input):
                    logic...

    """

    def __init_subclass__(cls, **kwargs):
        _inputs = getattr(cls._meta, "_inputs", [])
        _required_inputs = getattr(cls._meta, "_required_inputs", [])

        if _inputs or _required_inputs:
            if not isinstance(_inputs, (dict, list)) and _inputs:
                raise WrongUsage(f"dynamic inputs can be list or dict not{type(_inputs)}")
            if not isinstance(_required_inputs, (dict, list)) and _required_inputs:
                raise WrongUsage(
                    f"dynamic required inputs" f" can be list or dict not{type(_required_inputs)}"
                )

        parent_resolver_name = getattr(cls._meta, "_parent_resolver_name", None)
        if parent_resolver_name:
            _inputs = list_to_dict(_inputs)
            _required_inputs = list_to_dict(_required_inputs)

            parent_resolver = getattr(cls, parent_resolver_name, None)
            for name, ret_type in parent_resolver.base_resolver.annotations.items():
                if name != "return":
                    if is_optional(ret_type):
                        _inputs[name] = typing.get_args(ret_type)[0]
                    else:
                        _required_inputs[name] = ret_type

        if _inputs or _required_inputs:

            args = make_dataclass_helper(_required_inputs, _inputs)

            cls._meta.args = args
        else:
            cls._meta.args = []
        super().__init_subclass__()

    def field(self):
        raise NotImplementedError(
            "This mixin has to be mixed with either `DynamicRelayMutationMixin`,"
            " or `DynamicDefaultMutationMixin`"
        )

    async def afield(self):
        raise NotImplementedError()


class DynamicPayloadMixin:
    """
    A class that knows how to initialize strawberry
    payload from a list or dict
    and to merge parent payload to the given payload

    as dict { payload_name: payload_type }.
    as list [output_name,] -> defaults to String.
    or from the resolve_mutation return annotation.

    #usage:
        class SomeMutation(DynamicInputMixin, OutputMixin, MutationMixin):
                _outputs = [some, non, required, fields]
                _required_outputs = {some:str, required:int, fields:list}

                def resolve_mutation(self, input) -> SomeType:
                    logic...
    """

    def __init_subclass__(cls, **kwargs):
        _outputs = getattr(cls._meta, "_outputs", [])
        _required_outputs = getattr(cls._meta, "_required_outputs", [])

        if _outputs or _required_outputs:
            if not isinstance(_outputs, (dict, list, None)) and _outputs:
                raise WrongUsage(f"dynamic outputs can be list or dict not{type(_outputs)}")
            if not isinstance(_required_outputs, (dict, list, None)) and _required_outputs:
                raise WrongUsage(
                    f"dynamic required" f" outputs can be list or dict not{type(_required_outputs)}"
                )

        parent_resolver_name = getattr(cls._meta, "_parent_resolver_name", None)
        if parent_resolver_name:
            parent_resolver = getattr(cls, parent_resolver_name, None)
            new_dict = {}
            if isinstance(_outputs, list):
                # must create a dict for non str types
                for output in _outputs:
                    new_dict[output] = str
                _outputs = new_dict

            _outputs[parent_resolver_name + "Payload"] = parent_resolver.type

        if _outputs or _required_outputs:
            dc = dataclasses.make_dataclass(
                f"{cls.__name__}Outputs",
                [
                    ("success", bool),
                    *make_dataclass_helper(_required_outputs, _outputs),
                    (
                        "errors",
                        typing.Optional[ExpectedErrorType],
                        dataclasses.field(default=None),
                    ),
                ],
                namespace={"get": lambda self, attr: self.__getattribute__(attr)},
            )
            outputs = strawberry.type(dc)
        else:
            outputs = strawberry.type(OutputInterface)

        cls.output = outputs
        super().__init_subclass__()

    @property
    def field(self):
        raise NotImplementedError()(
            "This mimxin has to be mixed with either `DynamicRelayMutationMixin`,"
            " or `DynamicDefaultMutationMixin`"
        )


class DynamicRelayMutationMixin:
    def __init_subclass__(cls, **kwargs):
        @strawberry.mutation(description=cls.__doc__)
        def field(info: Info, input: cls._meta.inputs) -> cls.output:  # noqa: A002
            arguments = {f.name: getattr(input, f.name) for f in dataclasses.fields(input)}
            return cls.resolve_mutation(info, **arguments)

        if field.arguments[0].type is None:
            field.arguments.pop(0)  # if no input
        cls.field = field

        @strawberry.mutation(description=cls.__doc__)
        async def afield(info: Info, input: cls._meta.inputs) -> cls.output:  # noqa: A002
            arguments = {f.name: getattr(input, f.name) for f in dataclasses.fields(input)}
            return await sync_to_async(cls.resolve_mutation)(info, **arguments)

        if afield.arguments[0].type is None:
            afield.arguments.pop(0)  # if no input
        cls.afield = afield

    def resolve_mutation(self, info, **kwargs):
        raise NotImplementedError()(
            "This has method has to be implemented by the logic delegated mixin"
        )


class DynamicArgsMutationMixin:
    def __init_subclass__(cls, **kwargs):
        def field(info: Info, **kwargs) -> cls.output:
            return cls.resolve_mutation(info, **kwargs)

        field = hide_args_kwargs(field)
        field = strawberry.mutation(field, description=cls.__doc__)

        for arg_tuple in cls._meta.args:
            arg = create_strawberry_argument(arg_tuple[0], arg_tuple[0], arg_tuple[1])
            field.arguments.append(arg)
        cls.field = field

        async def afield(info: Info, **kwargs) -> cls.output:
            return await sync_to_async(cls.resolve_mutation)(info, **kwargs)

        afield = hide_args_kwargs(afield)
        afield = strawberry.mutation(afield, description=cls.__doc__)

        for arg_tuple in cls._meta.args:
            arg = create_strawberry_argument(arg_tuple[0], arg_tuple[0], arg_tuple[1])
            afield.arguments.append(arg)
        cls.afield = afield

    def resolve_mutation(self, info, *args, **kwargs):
        raise NotImplementedError()(
            "This has method has to be implemented by the logic delegated mixin"
        )



# Generated by Django 4.0.6 on 2022-08-02 19:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("gqlauth", "0005_rename_url_captcha_image"),
    ]

    operations = [
        migrations.AlterField(
            model_name="captcha",
            name="image",
            field=models.ImageField(
                editable=False, help_text="url for the captcha image", upload_to="captcha/%Y/%m/%d/"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gqlauth',
 'gqlauth.bases',
 'gqlauth.factory',
 'gqlauth.migrations',
 'gqlauth.user']

package_data = \
{'': ['*'],
 'gqlauth': ['templates/email/*', 'templates/email/images/*'],
 'gqlauth.factory': ['fonts/*']}

install_requires = \
['Pillow>=9.2.0,<10.0.0',
 'PyJWT>=1.7.1,<3.0',
 'django-admin-display>=1.3.0,<2.0.0',
 'packaging>=20.0,<30.0',
 'strawberry-django-jwt>=0.2.1,<0.3.0',
 'strawberry-graphql-django>=0.2.5,<4.0',
 'strawberry-graphql>=0.69.0,<1.0.0']

extras_require = \
{':python_version <= "3.7"': ['importlib-metadata>=1.7,<5.0']}

setup_kwargs = {
    'name': 'strawberry-django-auth',
    'version': '0.3.4.0',
    'description': 'Graphql authentication system with Strawberry for Django.',
    'long_description': "\n![Tests](https://github.com/nrbnlulu/strawberry-django-auth/actions/workflows/tests.yml/badge.svg)\n[![Pypi](https://img.shields.io/pypi/v/strawberry-django-auth.svg)](https://pypi.org/project/strawberry-django-auth/)\n[![contributions welcome](https://img.shields.io/badge/contributions-welcome-brightgreen.svg?style=flat)](https://github.com/nrbnlulu/strawberry-django-auth/blob/master/CONTRIBUTING.md)\n# Strawberry-django Auth\n[Django](https://github.com/django/django) registration and authentication with [Strawberry](https://strawberry.rocks/).\n\n## Demo\n\n![Demo Video](https://github.com/nrbnlulu/strawberry-django-auth/blob/main/demo.gif)\n\n## About\n### This Library is the strawberry version of ![Django-graphql-auth](https://github.com/pedrobern/django-graphql-auth/).\n\nAbstract all the basic logic of handling user accounts out of your app,\nso you don't need to think about it and can **get you up and running faster**.\n\nNo lock-in. When you are ready to implement your own code or this package\nis not up to your expectations , it's *easy to extend or switch to\nyour implementation*.\n\n\n### Docs can be found [here](https://nrbnlulu.github.io/strawberry-django-auth/)\n\n## Features\n\n* [x] Awesome docs!\n* [x] Captcha validation\n* [x] Async/Sync supported!\n* [x] Works with ==default or custom== user model\n* [x] JWT authentication <small>(with [Strawberry Django JWT](https://github.com/KundaPanda/strawberry-django-jwt))</small>\n* [x] User registration with email verification\n* [x] Add secondary email, with email verification too\n* [x] Resend activation email\n* [x] Retrieve/Update user\n* [x] Archive user\n* [x] Permanently delete user or make it inactive\n* [x] Turn archived user active again on login\n* [x] Track user status <small>(archived, verified, secondary email)</small>\n* [x] Password change\n* [x] Password reset through email\n* [x] Revoke user tokens on account archive/delete/password change/reset\n* [x] All mutations return `success` and `errors`\n* [x] Default email templates <small>(you will customize though)</small>\n* [x] Customizable, no lock-in\n* [x] Passwordless registration\n* [ ] Currently, only mutation\n\n\n### Full schema features\n\n```python\nimport strawberry\nfrom gqlauth.user import arg_mutations as mutations\n\n\n@strawberry.type\nclass AuthMutation:\n    register = mutations.Register.field\n    verify_account = mutations.VerifyAccount.field\n    resend_activation_email = mutations.ResendActivationEmail.field\n    send_password_reset_email = mutations.SendPasswordResetEmail.field\n    password_reset = mutations.PasswordReset.field\n    password_set = mutations.PasswordSet.field\n    password_change = mutations.PasswordChange.field\n    archive_account = mutations.ArchiveAccount.field\n    delete_account = mutations.DeleteAccount.field\n    update_account = mutations.UpdateAccount.field\n    send_secondary_email_activation = mutations.SendSecondaryEmailActivation.field\n    verify_secondary_email = mutations.VerifySecondaryEmail.field\n    swap_emails = mutations.SwapEmails.field\n    captcha = mutations.Captcha.field\n\n    # django-graphql-jwt authentication\n    # with some extra features\n    token_auth = mutations.ObtainJSONWebToken.field\n    verify_token = mutations.VerifyToken.field\n    refresh_token = mutations.RefreshToken.field\n    revoke_token = mutations.RevokeToken.field\n\n\nschema = strawberry.Schema(mutation=AuthMutation)\n```\n\n## Contributing\n\nSee [CONTRIBUTING.md](https://github.com/nrbnlulu/strawberry-django-auth/blob/master/CONTRIBUTING.md)\n",
    'author': 'Nir.J Benlulu',
    'author_email': 'nrbnlulu@gmail.com',
    'maintainer': 'Nir.J Benlulu',
    'maintainer_email': 'nrbnlulu@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
